/*
i-net software provides programming examples for illustration only, without warranty
either expressed or implied, including, but not limited to, the implied warranties
of merchantability and/or fitness for a particular purpose. This programming example
assumes that you are familiar with the programming language being demonstrated and
the tools used to create and debug procedures. i-net software support professionals
can help explain the functionality of a particular procedure, but they will not modify
these examples to provide added functionality or construct procedures to meet your
specific needs.
 i-net software 2003

  An example to demonstrate how to restore a broken Connection in the Background of the Application

*/
package samples;


import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import com.inet.pool.FailoverEvent;
import com.inet.pool.FailoverListener;
import com.inet.pool.PoolManager;

public class MonitoredFailover {
	
    
    public static void main(String[] args) throws Exception {
		testReconnect();
	}
    
    
    /**
     * To demonstrate how the connection failover works, disconnect from oracle server during the loop is running.
     * To interrupt the connection, restart the sql server service or remove the network cable. 
     * @throws Exception
     */
	private static void testReconnect() throws Exception {
		Connection conn = getConnection();
		//PoolManager.addFailoverListener(new TimedFailoverListener(20),conn);//try to reconnect for a defined time.
		PoolManager.addFailoverListener(new CounteredFailoverListener(20),conn);//try to reconnect for a defined count.
		PreparedStatement pst = conn.prepareStatement("select * from dual where ?=? and ?=?");
		pst.setInt(1,1);
		pst.setInt(2,1);
		pst.setInt(3,5);
		pst.setInt(4,5);
		int totalRuns=0;
		int errors=0;
		
		while(true){
		    try{
		        totalRuns++;
		        ResultSet rs=pst.executeQuery();
		        rs.next();
		        System.err.println(rs.getString(1));
		        System.err.println("runs: "+totalRuns+"\nerrors: " +errors);
		        Thread.sleep(1000);
		    }catch(SQLException e){
		        errors++;
		        System.err.println("[Test]SQLState: "+e.getSQLState()+" ErrorCode: "+e.getErrorCode()+"  "+e.getMessage());
		    }
		}
	}
	
	
	private static Connection getConnection()throws Exception{
		Class.forName("com.inet.pool.PoolDriver");
		DriverManager.setLogStream(System.out);
		return DriverManager.getConnection("jdbc:inetpool:inetora://localhost:1521:orcl","scott", "tiger");
	}
	
	
	/**
	 * A sample for a FailoverListener implementation.
	 * The attempt of reconnection will be done for a defined time. 
	 */
	public static class TimedFailoverListener implements FailoverListener{
	    private int seconds=0;
	    private long reconnectStartTime=0;
	    
	    
	    /**
	     * @param timeout in seconds
	     */
	    public TimedFailoverListener(int timeout) {
	        seconds = timeout;
        }
	    
	    
	    /**
         * @see com.inet.pool.FailoverListener#handleFailoverEvent(com.inet.pool.FailoverEvent)
         */
        public boolean handleFailoverEvent(FailoverEvent e) {
           switch(e.getEventType()){
           	case FailoverEvent.FO_BEGIN :
           	    reconnectStartTime = System.currentTimeMillis();
           		return true;
           	case FailoverEvent.FO_ERROR : 
           	    if((System.currentTimeMillis()-reconnectStartTime)*1000 < seconds){
           	        try{Thread.sleep(1000);}catch (InterruptedException ex) {}
           	        return true;
           	    }else{
           	        return false;	//Timeout
           	    }
           	case FailoverEvent.FO_END : 
           	    //do nothing
           	    return true;//for compiler, 
           	default :return true;//for compiler
           }
            
        }
        
	}
	
	
	/**
	 *A sample for a FailoverListener implementation. 
	 *The attempt of reconnection will be done for a defined count.
	 */
	public static class CounteredFailoverListener implements FailoverListener{
	    private int maxRetryCount=0;
	    private int retryCount=0;
	    
	    
	    /**
	     * @param number of attempts
	     */
	    public CounteredFailoverListener(int maxRetryCount) {
	        this.maxRetryCount = maxRetryCount;
        }
	    public CounteredFailoverListener() {
	        this.maxRetryCount = 50;
        }
	    
	    /** 
         * @see com.inet.pool.FailoverListener#handleFailoverEvent(com.inet.pool.FailoverEvent)
         */
        public boolean handleFailoverEvent(FailoverEvent e) {
           switch(e.getEventType()){
           	case FailoverEvent.FO_BEGIN :
           	    retryCount = 0;
           		return true;
           	case FailoverEvent.FO_ERROR : 
           	    if(retryCount++ < maxRetryCount){
           	        try{Thread.sleep(1000);}catch (InterruptedException ex) {}
           	        return true;
           	    }else{
           	        return false;	//reached maxRetryCount
           	    }
           	case FailoverEvent.FO_END : 
           	    //do nothing
           	    return true;
           	default :return true;
           }
        }
	}
}

